/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Adam Lock <adamlock@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

/**
 * @status UNDER_REVIEW
 */

interface nsIDOMWindow;


[scriptable, uuid(731C6C50-67D6-11d4-9529-0020183BF181)]
interface nsICommandHandlerInit : nsISupports
{
	attribute nsIDOMWindow window;
};

[scriptable, uuid(34A4FCF0-66FC-11d4-9528-0020183BF181)]
interface nsICommandHandler : nsISupports
{
	/*
	 * Execute the specified command with the specified parameters and return
	 * the result to the caller. The format of the command, parameters and 
	 * the result are determined by the acutal implementation.
	 */
    string exec(in string aCommand, in string aParameters);
	/*
	 * Query the status of the specified command with the specified parameters
	 * and return the result to the caller. The format of the command,
	 * parameters and the result are determined by the implementation.
	 */
    string query(in string aCommand, in string aParameters);
};

%{ C++
//	{3A449110-66FD-11d4-9528-0020183BF181} - 
#define NS_COMMANDHANDLER_CID \
{ 0x3a449110, 0x66fd, 0x11d4, { 0x95, 0x28, 0x0, 0x20, 0x18, 0x3b, 0xf1, 0x81 } }
#define NS_COMMANDHANDLER_CONTRACTID \
"@mozilla.org/embedding/browser/nsCommandHandler;1"
%}

